/***********    Program 3.2: Macro CORRTEST      ***************/
/*                                                             */
/* Macro CORRTEST executes the correlation test of randomness. */
/*                                                             */
/* Parameters:                                                 */
/* DATA    the name of the table containing a random variable. */
/* VAR     the name of the random variable.                    */
/* HLAG    the highest lag of correlation to be calculated.    */
/*         The macro calculates the correlation coefficients   */
/*         between subsequent random values with lags of 1, 2, */
/*         3, ... &HLAG. One correlation coefficient is        */
/*         calculated for each lag.                            */
/*                                                             */
/* Notes                                                       */
/* 1. The macro assumes that there is no missing value in the  */
/*    input table and it has at least HLAG number of rows.     */
/* 2. The name of the random variable must not start with      */
/*    'lag'.                                                   */
/*                                                             */
/***************************************************************/

%MACRO CORRTEST(DATA=,HLAG=,VAR=);

  /* Create a new table that has the original random value in  */
  /* column named 'lag0' and one column for each lag specified.*/
  /* For example                                               */
  /*                                                           */
  /* Orignal                                                   */
  /*  Table         New Table                                  */
  /*                                                           */
  /*    X       LAG0 LAG1 LAG2 LAG3                            */
  /*    -       ---- ---- ---- ----                            */
  /*    1          1    2    3    4                            */
  /*    2          2    3    4    5                            */
  /*    3          3    4    5    6                            */
  /*    4          4    5    6    .                            */
  /*    5          5    6    .    .                            */
  /*    6          6    .    .    .                            */
 
 DATA WORK(KEEP=LAG0-LAG&HLAG);
       SET &DATA NOBS=NOBS END=END;
       ARRAY LAGARRAY(*) LAG0-LAG&HLAG;
       RETAIN LAG0-LAG&HLAG;
       IF _N_<=&HLAG THEN LAGARRAY(_N_+1)=&VAR;
                       ELSE DO; DO I=1 TO &HLAG;
                                   LAGARRAY(I)=LAGARRAY(I+1);
                                END;
                                LAGARRAY(&HLAG+1)=&VAR;
                                OUTPUT;
                             END;
       IF END THEN DO; CALL SYMPUT('N',
                            COMPRESS(PUT(NOBS,BEST10.)));
                       DO J=0 TO &HLAG-2;
                          DO I=1 TO &HLAG-J;
                             LAGARRAY(I)=LAGARRAY(I+1);
                          END;
                          LAGARRAY(&HLAG-J+1)=.;
                          OUTPUT;
                       END;
                   END;
       RUN;

  /* calculate the correlation coefficients for each lag and   */
  /* save them.                                                */

  PROC CORR DATA=WORK OUTP=WORK(WHERE=(_TYPE_='CORR')) NOPRINT;
       VAR LAG0;
       WITH LAG1-LAG&HLAG;

  /* calculate the p-value for each correlation coefficient    */
  /* (because PROC CORR does not save it along with the        */
  /* correlation coefficient).                                 */

  DATA WORK(KEEP=PVALUE);
       SET WORK;
       RETAIN N &N;
       CORR=LAG0;
       N=N-1;           *** Number of values - degrees of freedom.;
       IF ABS(CORR)=1 THEN PVALUE=0;
                      ELSE PVALUE=2*(1-PROBT(ABS(CORR/
                           SQRT(1-(CORR*CORR))*SQRT(N-2)),N-2));
       OUTPUT;
       RUN;
  PROC FORMAT;
       VALUE SIGNIF 0.0000-0.0001='P<0.0001'
                    0.0001-0.001 ='P<0.001'
                    0.001 -0.01  ='P<0.01'
                    0.01  -0.05  ='P<0.05'
                    0.05  -0.10  ='P<0.10'
                    0.10  -0.15  ='P<0.15'
                    0.15  -1     ='P>0.15';

  /* summarize the results */

  PROC FREQ DATA=WORK;
       TABLE PVALUE;
       FORMAT PVALUE SIGNIF.;
       TITLE1 "Results of the Correlation Test";
       TITLE2 "-------------------------------";
       TITLE3 "# of Random Numbers: &N";
       TITLE4 "Lags Tested: 1 to &HORDER";
  RUN;
%MEND;
*************************************************************;
