
*************  Program 5.5: Macro Parking  *************************;

LIBNAME MYMACLIB 'subdirectory';   *** this is the location of;
OPTION SASMSTORE=MYMACLIB;         *** the macro library.;
/************************************************************/
/* Macro PARKING simulates the parking problem and returns  */
/* the number of cars that park along a given street.       */
/*                                                          */
/* Parameters                                               */
/* OUT       name of the dataset that captures the results  */
/*           of the simulations.                            */
/* STRLNGTH  the length of the street.                      */
/* CARLNGTH  the uniform length of a car.                   */
/* NSIMS     # of simulations.                              */
/* SEED      Seed of function RANUNI.                       */
/* MSG       # of simulations at which to display a         */
/*           a monitoring message in the LOG window.        */
/* LEVEL     MAIN or PARKING. It controls the level of      */
/*           detail in the output dataset. MAIN creates one */
/*           record per simulation. PARKING creates one     */
/*           record for every car parked.                   */
/* KEEP      NCARS or ALL. It controls the variables in the */
/*           output dataset. NCARS saves only variable      */
/*           NCARS (# of cars parked), ALL saves all        */
/*           auxiliary variables along with NCARS.          */
/************************************************************/

%MACRO PARKING(OUT=TEMP,STRLNGTH=10,CARLNGTH=1,
               NSIMS=1,SEED=123,MSG=0,LEVEL=MAIN,KEEP=NCARS);

 /* calculate the maximum number of cars that can park + 2. */
 /* do it in a data _null_ step and save the result in      */
 /* macro variable MAXNCARS using the SYMPUT CALL routine.  */
 /* macro variable &MAXNCARS will be used to dimension an   */
 /* array, which holds the locations of the parked cars.    */

 DATA _NULL_;
      IF &STRLNGTH.<&CARLNGTH.
         THEN MAXNCARS=0;
         ELSE MAXNCARS=2+INT(&STRLNGTH/&CARLNGTH);
      CALL SYMPUT('MAXNCARS',COMPRESS(PUT(MAXNCARS,16.)));
      STOP; RUN;

 /* if parameters are invalid, jump to the end of the       */
 /* macro and do nothing.                                   */

 %IF &MAXNCARS=0 %THEN %DO;
     %PUT Error: Invalid relative values for parameters 'STRLNGTH';
     %PUT Error: and 'CARLNGTH'.;
     %GOTO FINISH; %END;

 /* data step of the simulation. */

 DATA &OUT;

      /* The parameters are placed in the corresponding SAS   */
      /* variables through a RETAIN statement.                */

      LENGTH STRLNGTH CARLNGTH 4;
      RETAIN STRLNGTH &STRLNGTH CARLNGTH &CARLNGTH;
      RETAIN NSIMS &NSIMS SEED &SEED;

      /* array PRKGLOC stores the centrum of each car parked. */
      /* its dimension is the maximum number of cars that     */
      /* can park + 2 (see macro variable &MAXNCARS).         */
      /* the minimum parking location can be half the length  */
      /* of a car from the start of the street and the        */
      /* maximum parking location can be half the length of a */
      /* car from the end of the street.                      */
      /* to let our algorithm work, we park two imaginary     */
      /* cars just before and after the street.               */

      ARRAY PRKGLOC(&MAXNCARS) P1-P&MAXNCARS;
      LENGTH NCARS 4;
      %TMONST(MSG=&MSG)   *** start the time monitoring.;

      DO SIM=1 TO NSIMS;  *** the main LOOP of the simulations.;
         %TMON            *** display the monitoring message.;
         NCARS=0;         *** NCARS holds the number of cars;
                          *** currently parked.;

         /* set the parking locations of the two imaginary    */
         /* cars. All cars that the simulation parks will be  */
         /* parked between these two cars.                    */

         PRKGLOC(1)=-CARLNGTH/2;
         PRKGLOC(2)= STRLNGTH+CARLNGTH/2;

         /* NAVAILPS: # of available parking spaces. At the   */
         /* beginning, it is one, because the whole street    */
         /* is one continuous parking space.                  */

         NAVAILPS=1;

         /* LOOP that parks cars. It keeps parking until      */
         /* there is at least one available parking space.    */

         DO WHILE (NAVAILPS>0);

            /* Choose from the available parking spaces. */

            CURRPARK=1+INT(RANUNI(SEED)*NAVAILPS);

            /* find the possible parking spaces between the   */
            /* cars already parked and stop when the randomly */
            /* chosen parking space is reached.               */

            NPS=0;
            DO I=1 TO NCARS+1 WHILE (CURRPARK^=NPS);
               IF PRKGLOC(I)+2*CARLNGTH <= PRKGLOC(I+1) THEN DO;

                  /* the space between car i and car i+1 is    */
                  /* long enough to be a parking space.        */

                  NPS=NPS+1;   *** Keep counting the parking spaces.;

                  /* the randomly chosen parking space         */
                  /* (see above: CURRPARK) is found.           */

                  IF CURRPARK=NPS THEN DO;

                     /* choose the parking location within the */
                     /* parking space randomly. The parking    */
                     /* location is held in variable NEWPRKG.  */

                     NEWPRKG=PRKGLOC(I)+CARLNGTH+RANUNI(SEED)*
                             (PRKGLOC(I+1)-PRKGLOC(I)-2*CARLNGTH);

                     /* insert the car into the sequence of    */
                     /* parked cars. Push up all cars behind   */
                     /* it.                                    */

                     DO J=NCARS+2 TO I+1 BY -1;
                        PRKGLOC(J+1)=PRKGLOC(J);
                     END;
                     NCARS=NCARS+1;
                     PRKGLOC(I+1)=NEWPRKG;

                     /* update the number of available parking */
                     /* spaces. Is there enough space for      */
                     /* another car in front of and behind     */
                     /* this newly parked car?                 */

                     NAVAILPS=NAVAILPS-1;
                     IF PRKGLOC(I)+2*CARLNGTH<=PRKGLOC(I+1)
                        THEN NAVAILPS=NAVAILPS+1;
                     IF PRKGLOC(I+1)+2*CARLNGTH<=PRKGLOC(I+2)
                        THEN NAVAILPS=NAVAILPS+1;
                  END;
               END;
            END;

            /* output record after each parking. */

            %if %upcase(&level)=PARKING %then output;;
         end;

         /* output record after all cars are parked and no     */
         /* more cars can be parked.                           */

         %IF %UPCASE(&LEVEL)=MAIN %THEN OUTPUT;;
      END;
      %IF %UPCASE(&KEEP)=NCARS %THEN %STR(KEEP NCARS;);
                               %ELSE %STR(KEEP NCARS CARLNGTH STRLNGTH P1-P&MAXNCARS;);
      RUN;
 %FINISH:;
%MEND;
**************************************************************************************;
