
*****  Program 7.1  Simulating Structural Equation Models  *****;

/*******************************************************************/
/* this program conducts Monte Carlo simulation of SEM, and output */
/* parameter estimates and fit indices to a SAS system file named  */
/* named 'SEM_FITS'.  2 models are simulated in this program:      */
/* true, & misspecified models.                                    */
/*******************************************************************/

/******************************************************************/
%LET MC=200;   * # of Monte Carlo replications for each cell condition;
/******************************************************************/

LIBNAME SEM 'C:\SEM\SAS\';

     * -- to direct the SAS log to a disk file;
PROC PRINTTO LOG='C:\SEM\SAS\LOGFILE.TMP';

DATA A (TYPE=CORR);
_TYPE_='CORR';
INPUT X1 X2 Y1 Y2 Y3 Y4;
CARDS;
1.000000 . . . . .
 .536875 1.000000 . . . .
-.379164 -.290805 1.000000 . . .
-.414038 -.317552  .669246 1.000000 . .
-.375884 -.288290  .569164  .482667 1.000000 .
-.388047 -.297619  .456315  .529719  .672364 1.000000
;
             * obtain factor pattern matrix for later data generation;
PROC FACTOR N=6 OUTSTAT=FACOUT; 
DATA PATTERN; SET FACOUT;
  IF _TYPE_='PATTERN';
  DROP _TYPE_ _NAME_;
RUN;

%MACRO SEM_MC;  * start of monte carlo simulation macro 'SEM_MC';

                * do-loop for 2 conditions for true and misspecified models;
%DO MODEL = 1 %TO 2;
  %IF &MODEL=1 %THEN %DO; %LET MODL=TRU; %END;
  %IF &MODEL=2 %THEN %DO; %LET MODL=MIS; %END;

                    * do-loop for 2 estimation procedures;
%DO A = 1 %TO 2;
  %IF &A=1 %THEN %DO; %LET METHOD=MAX;  %END;
  %IF &A=2 %THEN %DO; %LET METHOD=GLS;  %END;

                    * do-loop for 4 sample size conditions;
%DO B = 1 %TO 4;
  %IF &B=1 %THEN %DO; %LET SMPLN=100;  %END;
  %IF &B=2 %THEN %DO; %LET SMPLN=200;  %END;
  %IF &B=3 %THEN %DO; %LET SMPLN=500;  %END;
  %IF &B=4 %THEN %DO; %LET SMPLN=1000; %END;

%DO C=1 %TO &MC;     * do-loop for the number of replications in each cell;

PROC IML;                    * use SAS PROC iml for data generation;
   USE PATTERN;              * use the factor pattern matrix;
   READ ALL VAR _NUM_ INTO F;
  F=F`;

           * diagnoal matrix containing variances for 6 variables;

 VAR={10   0     0      0       0        0,
      0 4.25     0      0       0        0,
      0    0 12.27      0       0        0,
      0    0     0 9.2868       0        0,
      0    0     0      0 12.9047        0,
      0    0     0      0       0 9.807807};

STD=SQRT(VAR);            * matrix containing stds for the 6 variables;
X=RANNOR(J(&SMPLN,6,0));  * generate 6 random normal variables;
XT=X`;                    * transpose the data matrix for multiplication;

                  * transform uncorrelated variables to correlated ones;
XTCORR=F*XT;

        *transform the scale of the variables
         (from std=1 to std=specified above);

XTSTD=STD*XTCORR;

             * transpose the data matrix back;
XY=XTSTD`;
RUN;

             * create SAS data set 'DAT';
CREATE DAT FROM XY[COLNAME={X1 X2 Y1 Y2 Y3 Y4}];
APPEND FROM XY;
                         * implement the true model;
                         * output the model fitting results to data 'SEMOUT';
%IF &MODEL=1 %THEN %DO;

PROC CALIS DATA=DAT METHOD=&METHOD COV OUTRAM=SEMOUT NOPRINT;
  LINEQS
    X1 =     FK  +   EX1,
    X2 = LX2 FK  +   EX2,
    Y1 =     FE1 +   EY1,
    Y2 = LY2 FE1 +   EY2,
    Y3 =     FE2 +   EY3,
    Y4 = LY4 FE2 +   EY4,
    FE1= GA1 FK  +   DE1,
    FE2= GA2 FK  +  BE1 FE1 + DE2;
STD
 FK EX1 EX2 EY1 EY2 EY3 EY4 DE1 DE2 =
 VFK VEX1 VEX2 VEY1 VEY2 VEY3 VEY4 VDE1 VDE2;
COV
  EY1 EY3 = C_EY13, EY2 EY4 = C_EY24;
RUN;

DATA SEMOUT; SET SEMOUT;  * keep only the relevant results;
   KEEP _NAME_ _ESTIM_;
RUN;

                          * transpose the SAS data 'SEMOUT' to 'NEWFITS';
PROC TRANSPOSE DATA=SEMOUT OUT=NEWFITS LET;

   DATA NEWFITS;  SET NEWFITS;    * add simulation design information;
   MTHD="&METHOD"; MODEL="&MODL";

                          * keep the desired model fit indices;

   KEEP MODEL MTHD N FIT NPARM DF CHISQUAR P_CHISQ CHISQNUL GFI AGFI
        RMSEAEST COMPFITI BB_NONOR BB_NORMD BOL_RHO1 BOL_DEL2 CENTRALI;

		        * append results of each run to a SAS system file;

PROC APPEND BASE=SEM.SEM_FITS FORCE;

%END;      * end of implementing the true model;

                       * implement misspecified model;
%IF &MODEL=2 %THEN %DO;

PROC CALIS DATA=DAT METHOD=&METHOD COV OUTRAM=SEMOUT NOPRINT;
  LINEQS
    X1 =    FK  +   EX1,
    X2 = LX2 FK  +   EX2,
    Y1 =     FE1 +   EY1,
    Y2 = LX2 FE1 +   EY2,
    Y3 =     FE2 +   EY3,
    Y4 = LY4 FE2 +   EY4,
    FE1= GA1 FK  +   DE1,
    FE2= 0   FK  +   BE1 FE1 + DE2;    * misspecification: GA2 fixed;
                                       * constrained: LY2=LX2;
STD
 FK EX1 EX2 EY1 EY2 EY3 EY4 DE1 DE2 =
 VFK VEX1 VEX2 VEY1 VEY2 VEY3 VEY4 VDE1 VDE2;

        * mispecification: error covariances fixed to zeros;
*COV
  EY2 EY4 = C_EY24, EY1 EY3 = C_EY13;
RUN;


DATA SEMOUT; SET SEMOUT;
   KEEP _NAME_ _ESTIM_;
RUN;

PROC TRANSPOSE DATA=SEMOUT OUT=NEWFITS LET;
   DATA NEWFITS;   SET NEWFITS;   * add simulation design information;
   MTHD="&METHOD"; MODEL="&MODL";
   KEEP MODEL MTHD N FIT NPARM DF CHISQUAR P_CHISQ CHISQNUL GFI AGFI
        RMSEAEST COMPFITI BB_NONOR BB_NORMD BOL_RHO1 BOL_DEL2 CENTRALI;

PROC APPEND BASE=SEM.SEM_FITS FORCE;

%END;           * end of implementing the misspecified model;

%END;           * close the do-loop for replications in each cell;
%END;           * close the do-loop for sample size conditions;
%END;           * close the do-loop for estimation procedure;
%END;           * close the do-loop for model specification conditions;
%MEND SEM_MC;   * end of simulation macro 'SEM_MC';
%SEM_MC;        * running the macro 'SEM_MC';
RUN;

PROC PRINTTO PRINT=PRINT;  * direct output to SAS output window;
RUN;

/*       * descriptive analysis for the 9 descriptive model fit indices; 
DATA D2; SET SEM.SEM_FITS;
PROC SORT; BY MODEL MTHD N;
PROC MEANS MEAN STD MAX MIN; 
   BY MODEL MTHD N;
   VAR GFI AGFI RMSEAEST COMPFITI BB_NONOR BB_NORMD BOL_RHO1 BOL_DEL2 CENTRALI;
RUN;
*/
