
******  Program 7.2  Simulating Classification Error Rates of PDA and LR  *****;

/* This program conducts Monte Carlo simulation for comparing classification
   error rates from linear discriminant function and from logistic regression
   for two-group situation.  SAS PROC DISCRIM and SAS PROC LOGISTIC are used
   for group membership classification.  The results of classification error
   rates for the respective two groups and the overall error rate from both 
   PROC DICSCRIM and PROC LOGISTIC are saved in an external SAS system file
   for future analyses.
*/

LIBNAME PDA_LR 'C:\PDA_LR\SAS';

    * -- to direct the SAS log to an external file;
PROC PRINTTO LOG='C:\PDA_LR\SAS\LOGFILE.TMP';
RUN;

DATA D1(TYPE=CORR);
  INPUT _TYPE_ $ _NAME_ $ X1-X3;
  CARDS;
CORR X1  1 .  .
CORR X2  0.30  1  .
CORR X3  0.50  0.40  1
;
              *obtain factor pattern matrix for later data generation;
PROC FACTOR N=3 OUTSTAT=FACTOUT; 
DATA PATTERN;  SET FACTOUT;
     IF _TYPE_ = 'PATTERN';
     DROP _TYPE_ _NAME_;
RUN;

%MACRO PDA_LR;          * start of simulation macro;

%DO A = 1 %TO 2;        * A=1: equal group covariance, A=2: unequal;

                        * specify four pseudo-population sizes (N1+N2);
                        * specify sample size conditions (N);
%DO B = 1 %TO 4;
  %IF &B=1 %THEN %DO; %LET N1=600;  %LET N2=600;  %LET N=60;   %END;
  %IF &B=2 %THEN %DO; %LET N1=1000; %LET N2=1000; %LET N=100;  %END;
  %IF &B=3 %THEN %DO; %LET N1=2000; %LET N2=2000; %LET N=200;  %END;
  %IF &B=4 %THEN %DO; %LET N1=4000; %LET N2=4000; %LET N=400;  %END;

%DO NITER=1 %TO 1000;     * specify the # of replications within each cell;

PROC IML;          
  USE PATTERN;           * use the factor pattern matrix;
  READ ALL VAR _NUM_ INTO FT;
  F=FT`;
  MEAN1={5 5 5};         * specify two mean vectors for the two populations;
  MEAN2={9 9 9};

  VAR={4 0 0,
       0 4 0,
       0 0 4};           * specify the common variances of the 3 variables;

                         * specify equal (&A=1) and  
                           unequal (&A=2) covariance conditions;
IF &A=1 THEN VAR1=VAR;   
  ELSE VAR1=(2/5)*VAR;
IF &A=1 THEN VAR2=VAR;
  ELSE VAR2=(8/5)*VAR;

************ GROUP 1 DATA **********;

G1DATA=RANNOR(J(&N1,3,0));  * generate 3 random normal variables;
G1DATAT=G1DATA`;            * transpose the data matrix for multiplication;
G1DATAT=F*G1DATAT;          * transform uncorrelated variables to correlated;

STD1=SQRT(VAR1);            * transform the variables to specified scales;
G1DATAT=(STD1*G1DATAT)`;
X1=G1DATAT[,1] + MEAN1[,1];
X2=G1DATAT[,2] + MEAN1[,2];
X3=G1DATAT[,3] + MEAN1[,3];
GROUP1=J(&N1,1,1);          * assigning group number;
G1DATA=GROUP1||X1||X2||X3;

******** GROUP 2 DATA ********************;

G2DATA=RANNOR(J(&N2,3,0));  * generate 3 random normal variables;
G2DATAT=G2DATA`;            * transpose the data matrix for multiplication;
G2DATAT=F*G2DATAT;          * transform uncorrelated variables to correlated;

STD2=SQRT(VAR2);            * transform variables to specified scales;
G2DATAT=(STD2*G2DATAT)`;
X1=G2DATAT[,1] + MEAN2[,1];
X2=G2DATAT[,2] + MEAN2[,2];
X3=G2DATAT[,3] + MEAN2[,3];
GROUP2=J(&N2,1,2);          * assigning group number;
G2DATA=GROUP2||X1||X2||X3;
RUN;

G12DATA=G1DATA//G2DATA;     * combine data of two groups;

CREATE DATA FROM G12DATA[COLNAME={GROUP X1 X2 X3}];
APPEND FROM G12DATA;

      * draw a random sample of size N from the pseudo-population;

DATA TEMP; SET DATA;
  RANNO=RANUNI(0);
PROC SORT; BY RANNO;
DATA SMPLDAT; SET TEMP(OBS=&N); DROP RANNO;

               * direct SAS PROC DISCRIM output to an external file;

FILENAME NEWOUT 'C:\PDA_LR\SAS\OUTFILE';
PROC PRINTTO PRINT=NEWOUT NEW;

       * PROC DISCRIM, requesting jackknife procedure (CROSSVALIDATE);

PROC DISCRIM CROSSVALIDATE DATA=SMPLDAT;
  CLASS GROUP;
  PRIORS PROPORTIONAL;
  VAR X1 X2 X3;
RUN;

PROC PRINTTO PRINT=PRINT;  * direct SAS output back to SAS output window;
RUN;

  * read in the external file containing output from PROC DISCRIM;
  * read in jackknifed classification error rates for two groups and combined;

DATA PDAERROR; INFILE NEWOUT; 
  INPUT WORD1 $ @;
    IF WORD1='Rate' THEN DO;  
           INPUT DAG1_ERR DAG2_ERR DA_ERR;
           KEEP DAG1_ERR DAG2_ERR DA_ERR;
           OUTPUT;
    END;
RUN;

DATA PDAERROR; SET PDAERROR;     * keep the jackknifed errors only;
N=_N_;
IF N=2;  DROP N;
RUN;

       * direct PROC LOGISTIC output to an external file for later use;

FILENAME NEWOUT 'C:\PDA_LR\SAS\OUTFILE';
PROC PRINTTO PRINT=NEWOUT NEW;

       * conduct PROC LOGISTIC analysis for the sample data;
       * requesting classification table;
       * with probability = 0.5 as the cut-off point for classification;

PROC LOGISTIC DATA=SMPLDAT;
  MODEL GROUP=X1 X2 X3/CTABLE PPROB=.5;
RUN;

       * direct SAS output back to SAS output window;
PROC PRINTTO PRINT=PRINT; RUN;

 * from the external file containing output from PROC LOGISTIC;
 * read in the following:
      (a) overall correct classification rate (LRRATE),
      (b) GROUP 1 correct classification rate (G1RATE), 
      (c) GROUP 2 correct classification rate (G2RATE);
 * construct and keep the classification error rates for 
            (i) GROUP 1, (ii) GROUP 2, and (iii) overall error rate;

DATA LRERROR; INFILE NEWOUT;
  INPUT PRIOR $ @;
    IF PRIOR='0.500' THEN DO;
       INPUT V1 V2 V3 V4 LRRATE G1RATE G2RATE;
             LRG1_ERR=1-G1RATE/100;
             LRG2_ERR=1-G2RATE/100;
             LR_ERR=1-LRRATE/100;
       KEEP LRG1_ERR LRG2_ERR LR_ERR;
       OUTPUT;
    END;

     * merge error rates from PDA and LR;
     * add simulation study design information;
     * append the results from each sample to a SAS system file "RESULTS";

DATA PDA_LR;
  MERGE PDAERROR LRERROR;
  N=&N; IF &A=1 THEN COV='EQUAL'; ELSE IF &A=2 THEN COV='NO_EQ';
PROC APPEND BASE=PDA_LR.RESULTS FORCE;
RUN;

%END;          * end of do-loop for replications in each cell;
%END;          * end of do-loop for each sample size condition;
%END;          * end of do-loop for each covariance equality condition;
%MEND PDA_LR;  * end of simulation macro;
%PDA_LR;       * run the simulation macro;
RUN;


DATA A; SET PDA_LR.RESULTS;
PROC PRINT;
RUN;
/*
PROC SORT; BY COV N;
PROC MEANS MEAN STD MIN MAX; BY COV N;
  VAR DAG1_ERR LRG1_ERR DAG2_ERR LRG2_ERR DA_ERR LR_ERR;
TITLE1 'Average Classification Error Rates';
TITLE2 'Comparison of Discriminant Analysis and Logistic Regression';
TITLE3 'For GROUP 1, GROUP 2, and Overall';
RUN;
*/
