
************    Program 8.1 Macro (%EX1) for Simulating  *******************;
*******     Default Risk for Diversified Holding Company *******************;

/*****************************************************************************/
/* Macro EX1 calculates the default probability of a holding company.        */
/*                                                                           */
/* Parameters                                                                */
/* PARM      the name of a special file of the default probabilities, the    */
/*           dividend payments and the default correlation matrix of the     */
/*           subsidiaries. It is similar to a _TYPE_=CORR file. The first    */
/*           row corresponds to the default probabilities of the             */
/*           subsidiaries, the second to the dividend payments of the        */
/*           subsidiaries and the rest forms the lower half of the           */
/*           correlation matrix of the subsidiaries' default rates. See the  */
/*           example below.                                                  */
/* NSUBS     the number of subsidiaries.                                     */
/* DEFPOINT  percent of dividend over which default happens.                 */
/* NSIMS     number of simulations.                                          */
/* OUT       name of the output data set, which contains the default flag    */
/*           and the lost dividend.                                          */
/*                                                                           */
/* Example for creating an input file                                        */
/*                                                                           */
/* DATA SIMPAR;                                                              */
/*      INPUT _NAME_ $ _TYPE_ $ SUB1-SUB7;                                   */
/*      CARDS;                                                               */
/*      PNC   0.05     0.05     0.10     0.05     0.05     0.20     0.01     */
/* .    DIV   0.129630 0.060847 0.243386 0.034392 0.238095 0.026455 0.267196 */
/* SUB1 CORR  1.00      .        .        .        .        .        .       */
/* SUB2 CORR  0.10     1.00      .        .        .        .        .       */
/* SUB3 CORR  0.10     0.10     1.00      .        .        .        .       */
/* SUB4 CORR  0.10     0.10     0.10     1.00      .        .        .       */
/* SUB5 CORR  0.10     0.10     0.10     0.10     1.00      .        .       */
/* SUB6 CORR  0.00     0.00     0.00     0.00     0.00     1.00      .       */
/* SUB7 CORR  0.10     0.10     0.10     0.10     0.10     0.00     1.00     */
/* ;                                                                         */
/*      RUN;                                                                 */
/*****************************************************************************/

%MACRO EX1(PARM=,NSUBS=,DEFPOINT=,NSIMS=,OUT=,SEED=123);

 /* create a file for generating the normally distributed correlated     */
 /* random numbers for the probabilities of subsidiary defaults. This    */
 /* file will be input into macro %RMNC.                                 */

 DATA TEMP(TYPE=CORR DROP=I);
      SET &PARM;
      ARRAY SUB(&NSUBS) SUB1-SUB&NSUBS;
      IF _N_=1 THEN DO;       * add mean of zero for each subsidiary;
                        DO I=1 TO &NSUBS;
                           SUB(I)=0;
                        END;
                        _TYPE_='MEAN'; OUTPUT;

                        /* add standard deviation of one for each subsidiary. */

                        DO I=1 TO &NSUBS;
                           SUB(I)=1;
                        END;
                        _TYPE_='STD'; OUTPUT;

                        /* add number of simulations for each subsidiary. */

                        DO I=1 TO &NSUBS;
                           SUB(I)=&NSIMS;
                        END;
                        _TYPE_='N'; OUTPUT;
                    END;
      IF _TYPE_='CORR' THEN OUTPUT;
      RUN;

/* call macro %RMNC to generate correlated, normally distributed */
/* random numbers to simulate the defaults of the subsidiaries.  */

 %RMNC(DATA=TEMP,OUT=PROBS,SEED=&SEED)

 /* data step of the simulation */

 DATA &OUT;
      ARRAY PNC(&NSUBS) PNC1-PNC&NSUBS;   * array of default probabilities ;
      ARRAY DIV(&NSUBS) DIV1-DIV&NSUBS;   * array of dividends ;
      ARRAY SUB(&NSUBS) SUB1-SUB&NSUBS;

      /* load the default probability thresholds and the dividend */
      /* payments of all subsidiaries into the arrays above.      */

      DO WHILE (NOT EOFPARM);
         SET &PARM END=EOFPARM;
         IF _TYPE_='PNC' THEN DO; DO I=1 TO &NSUBS;

                                     /* transform the probability into  */
                                     /* a standard normal distribution. */

                                     PNC(I)=PROBIT(SUB(I));
                                  END;
                              END;
         IF _TYPE_='DIV' THEN DO; DO I=1 TO &NSUBS;
                                     DIV(I)=SUB(I);
                                  END;
                              END;
      END;

      /* go through each set of default probabilities generated above. */

      DO WHILE (NOT EOFSIM);
         SET PROBS END=EOFSIM;
         DIVLOSS=0;
         DO I=1 TO &NSUBS;

            /* does a subsidiary default? If does, add its forgone */
            /* dividend to the losses (variable DIVLOSS).          */

            IF SUB(I)<PNC(I) THEN DIVLOSS=DIVLOSS+DIV(I);
         END;

         /* does the dividend loss exceed the default point? */
         /* if yes, mark this draw as 'default'.             */

         IF DIVLOSS>&DEFPOINT THEN DEFAULT=1;
                              ELSE DEFAULT=0;
         OUTPUT;
      END;
      STOP;
      KEEP DEFAULT DIVLOSS;
      RUN;

 /* the default rate (the mean statistic of variable DEFAULT) and the */
 /* characteristics of the dividend losses (whether or not the draw   */
 /* results a default) are calculated by PROC MEANS.                  */

 PROC MEANS DATA=&OUT N MEAN STD MIN MAX;
      VAR DEFAULT DIVLOSS;
      RUN;
 %MEND;
**************************************************************************;
