********  Program 8.3 Monte Carlo Simulation for Portfolio Credit Risk  *******;
/*****************************************************************************/
/* Macro EX2 performs VAR-calculation from a set of pure credit events.      */
/*                                                                           */
/* Parameters                                                                */
/* PORTFOLIO the name of a file defining a credit portfolio of seven risk    */
/*           categories. The file has one variables: EXPOSURE, which is      */
/*           numeric and defines the exposure in whole dollar for each risk  */
/*           category. The number of risk categories, i.e. the number of     */
/*           observations, must be seven and they must be listed in          */
/*           increasing order of risk. See example below.                    */
/* NYEARS    the number of years, which determines the time horizon of the   */
/*           simulation.                                                     */
/* NSIMS     number of simulations.                                          */
/* OUT       name of the output data set, which contains the loss for each   */
/*           simulation and the number of high-default years.                */
/* SEED      Seed of the random number generator functions.                  */
/*                                                                           */
/* Example for creating the input file for the portfolio:                    */
/*                                                                           */
/* DATA PORTF(KEEP=EXPOSURE);                                                */
/*      INPUT RATING $ EXPOSURE;                                             */
/*      CARDS;                                                               */
/*      Aaa   10000   - Risk category 'Aaa' has the lowest level of risk.    */
/*      Aa    10000                                                          */
/*      A     10000                                                          */
/*      Baa   10000                                                          */
/*      Ba    0                                                              */
/*      B     0                                                              */
/*      C     0       - Risk category 'C' has the highest level of risk.     */
/*      ;                                                                    */
/*      RUN;                                                                 */
/*****************************************************************************/

%MACRO EX2(PORTFOLIO=,NYEARS=,NSIMS=,OUT=,SEED=123);

 DATA &OUT(KEEP=LOSS HIGHDEFYEAR);

      /* supply the parameters of the simulation. */

      RETAIN HISTDEFRATE 0.035   * historical average default rate; 
             THRESHOLD   0.06;   * threshold for 'high-default' year;

      ARRAY MIGR(2,7,8) MIGR1-MIGR112 (

            /* Migration matrix of "normal" years */
            /* From To:    Aaa    Aa     A      Baa    Ba     B      C      Default */
            /*             (1)    (2)    (3)    (4)    (5)    (6)    (7)    (8)     */

            /* Aaa (1) */  0.9168 0.0761 0.0069 0.0000 0.0002 0.0000 0.0000 0.0000
            /* Aa  (2) */  0.0143 0.9070 0.0746 0.0029 0.0009 0.0001 0.0000 0.0002
            /* A   (3) */  0.0007 0.0235 0.9187 0.0502 0.0055 0.0012 0.0001 0.0001
            /* Baa (4) */  0.0004 0.0026 0.0568 0.8786 0.0515 0.0077 0.0008 0.0014
            /* Ba  (5) */  0.0002 0.0005 0.0049 0.0547 0.8581 0.0655 0.0042 0.0119
            /* B   (6) */  0.0001 0.0003 0.0013 0.0046 0.0693 0.8398 0.0209 0.0637
            /* C   (7) */  0.0000 0.0000 0.0000 0.0071 0.0218 0.0450 0.6740 0.2520

            /* Migration matrix of "high-default" years */
            /* From To:    Aaa    Aa     A      Baa    Ba     B      C      Default */
            /*             (1)    (2)    (3)    (4)    (5)    (6)    (7)    (8)     */

            /* Aaa (1) */  0.9100 0.0600 0.0275 0.0025 0.0000 0.0000 0.0000 0.0000
            /* Aa  (2) */  0.0045 0.8453 0.1206 0.0132 0.0082 0.0032 0.0030 0.0020
            /* A   (3) */  0.0000 0.0051 0.8892 0.0736 0.0094 0.0057 0.0120 0.0050
            /* Baa (4) */  0.0000 0.0061 0.0377 0.8439 0.0512 0.0161 0.0350 0.0100
            /* Ba  (5) */  0.0000 0.0000 0.0066 0.0351 0.8387 0.0855 0.0066 0.0275
            /* B   (6) */  0.0000 0.0000 0.0000 0.0010 0.0400 0.8400 0.0133 0.1057
            /* C   (7) */  0.0000 0.0000 0.0000 0.0100 0.0200 0.1250 0.4700 0.3750
            );

      /* Define the distribution of loss in the event of default (LIED) as a    */
      /* stepwise function. Array 'LIED' contains the discrete loss values from */
      /* 1.25% to 98.75%, and array 'LIEDPROB' provides the probabilities, by   */
      /* which those losses happen. See Asarnow and Edwards.                    */

      ARRAY LIED(21)     (0.0125 0.05   0.10   0.15   0.20   0.25   0.30   0.35
                          0.40   0.45   0.50   0.55   0.60   0.65   0.70   0.75
                          0.80   0.85   0.90   0.95   0.9875);
      ARRAY LIEDPROB(20) (0.1237 0.1410 0.1069 0.0708 0.0450 0.0394 0.0427 0.0405
                          0.0450 0.0247 0.0225 0.0337 0.0259 0.0326 0.0281 0.0225
                          0.0247 0.0236 0.0225 0.0236       );

      ARRAY PROB(7) PROB1-PROB7;                /* temporary array   */
      ARRAY INITPORTF(7) INITPORTF1-INITPORTF7; /* initial portfolio */
      ARRAY CURPORTF(7)  CURPORTF1 -CURPORTF7;  /* current portfolio */
      ARRAY NEWPORTF(7)  NEWPORTF1 -NEWPORTF7;  /* new portfolio after a migration */

      /* load the input portfolio into array 'initportf'. */

      DO RATING=1 TO 7;
         SET &PORTFOLIO POINT=RATING;
         INITPORTF(RATING)=EXPOSURE;
      END;

      /* main loop of the simulations */

      DO SIM=1 TO &NSIMS;

         /* start with the input portfolio, move */
         /* it into the current portfolio.       */

         DO RATING=1 TO 7;
            CURPORTF(RATING)=INITPORTF(RATING);
         END;

         /* variable 'LOSS' summarizes the losses, set it to zero initially. */

         LOSS=0;

         /* variable 'HIGHDEFYEAR' keeps track of the */
         /* number of years with high default rate.   */

         HIGHDEFYEAR=0;

         /* migrate the portfolio through &NYEARS. */

         DO YEAR=1 TO &NYEARS;

            /* array 'NEWPORT' has the current portfolio after each annual */
            /* migration. Set it to zero before each yearly migration, and */
            /* then collect the exposures by the risk categories to which  */
            /* they migrate.                                               */

            DO RATING=1 TO 7;
               NEWPORTF(RATING)=0;
            END;

            /* determine the type of year, i.e. 'Normal' (REGIME=1) */
            /* or 'High-default' (REGIME=2).                        */

            MACROECO=RANBIN(&SEED,100,HISTDEFRATE)/100;
            IF MACROECO>THRESHOLD THEN REGIME=2;   * high-default year;
                                  ELSE REGIME=1;   * normal year;
            IF REGIME=2 THEN HIGHDEFYEAR=HIGHDEFYEAR+1;

            /* take one risk category of the portfolio. */

            DO RATING=1 TO 7;

               /* load the migration percentages of the given year-type and */
               /* rating into a temporary array to be used for each dollar  */
               /* of the current rating category.                           */

               DO I=1 TO 7;
                  PROB(I)=MIGR(REGIME,RATING,I);
               END;

               /* go through every exposure of a given rating category. */

               DO P=1 TO CURPORTF(RATING);

                  /* take a random draw from the migration probabilities */
                  /* to determine the new risk category of the exposure. */

                  NEWRATING=RANTBL(&SEED,OF PROB(*));

                  /* if the exposure migrates to risk category '8', it goes */
                  /* into default. Draw a random loss-value from the        */
                  /* empirical loss distribution. If the migration does     */
                  /* not result in default, assign the exposure to the new  */
                  /* risk category.                                         */

                  IF NEWRATING=8 THEN LOSS=LOSS+LIED(RANTBL(&SEED,OF LIEDPROB(*)));
                                 ELSE NEWPORTF(NEWRATING)=NEWPORTF(NEWRATING)+1;
               END;

            END;

            /* make the new portfolio the current one, so the */
            /* next annual migration can be performed.        */

            DO RATING=1 TO 7;
               CURPORTF(RATING)=NEWPORTF(RATING);
            END;
         END;
         OUTPUT;
      END;
      STOP;
      RUN;

 /* check the distribution of the number of years with high-default rate. */

 PROC FREQ DATA=&OUT;
      TABLE HIGHDEFYEAR;
      TITLE 'Distribution of the Number of High-Default Years';
      RUN;

 /* describe the loss distribution and determine certain percentiles of it. */

 PROC UNIVARIATE DATA=&OUT NOPRINT;
      VAR LOSS;
      OUTPUT OUT=RES N=N MEAN=MEAN STD=STD MAX=MAX MIN=MIN MEDIAN=MEDIAN
                     Q1=Q1 Q3=Q3 P90=P90 P95=P95 P99=P99;
      TITLE F=SWISS H=1.5
            "Distribution of Losses (Number of Years=&NYEARS., Number of Simulations=&NSIMS.)";
      run;

 /* print the results in a comprehensive vertical layout. */

 PROC TRANSPOSE DATA=RES OUT=TRES;
      RUN;
 PROC PRINT DATA=TRES LABEL NOOBS;
      VAR _LABEL_ COL1;
      LABEL _LABEL_='STATISTIC' COL1='LOSS ($)';
      FORMAT COL1 10.0;
      RUN;

 /* draw the graph of the ordered losses. Set up the labels */
 /* for the two reference lines at 95 and 99 percentiles.   */

 DATA ANNO;
      SET RES;
      FUNCTION='LABEL';  STYLE='DUPLEX'; SIZE=1.2;
      XSYS='2'; YSYS='2';
      Y=P95; X=10; TEXT='95% - '||COMPRESS(PUT(P95,10.0));
      POSITION='F'; OUTPUT;
      Y=P99; X=50; TEXT='99% - '||COMPRESS(PUT(P99,10.0));
      POSITION='C'; OUTPUT;
      CALL SYMPUT('VREF95',PUT(P95,BEST10.));
      CALL SYMPUT('VREF99',PUT(P99,BEST10.));
      RUN;
 PROC SORT DATA=&OUT OUT=TEMP;
      BY LOSS;
 DATA TEMP;
      SET TEMP;
      PERCENT=100*(_N_/&NSIMS);
      RUN;
 PROC GPLOT DATA=TEMP ANNOTATE=ANNO;
      PLOT LOSS*PERCENT / VAXIS=AXIS1 HAXIS=AXIS2 VREF=&VREF95 &VREF99;
      SYMBOL1 I=JOIN V=NONE W=1;
      AXIS1 LABEL=(A=90 R=0 F=SWISS H=1.5 'Portfolio Loss ($)')
            VALUE=(H=1.2 F=SWISS) MINOR=(N=1) OFFSET=(0,0);
      AXIS2 LABEL=(F=SWISS H=1.5 'Percentile')
            VALUE=(H=1.2 F=SWISS) ORDER=0 TO 100 BY 10 MINOR=(N=1) OFFSET=(0,0);
      RUN;
      QUIT;
%MEND;
********************************************************************************;
