
******   Program 8.6 VaR Estimation for Portfolio Market Risk   **********;
/*****************************************************************************/
/* Macro EX3 evaluates the risk associated with the change in the market     */
/* value of a portfolio due to changes in market conditions.                 */
/*                                                                           */
/* Parameters                                                                */
/* FACTORS   a SAS file containing the historical time series of the key     */
/*           factors. (For example S&P 500 Index, Yield to Maturity on the   */
/*           90-Day Treasury Bill and US Dollar/Japanese Yen Exchange Rate.) */
/*           The factors must be named FACTOR1, FACTOR2,... The file can     */
/*           have any number of factors, but it must not contain any other   */
/*           variables.                                                      */
/* PORT      a SAS file describing the portfolio. Every observation is an    */
/*           asset and the variables are VAR (the variance of the security), */
/*           and LOAD1, LOAD2,... (the loadings of the securities along the  */
/*           factors given in the file defined by parameter FACTORS).        */
/* WINDOW    the length of the window to estimate a variance matrix of the   */
/*           factors. The macro will take WINDOW number of consecutive       */
/*           observations, starting at a randomly selected observation, from */
/*           the historical time series of factors to determine a variance   */
/*           matrix.                                                         */
/* NSIMS     number of simulations.                                          */
/* OUT       name of the output data set, which contains the loss of the     */
/*           portfolio (variable LOSS) for each simulation.                  */
/* SEED      Seed of the random number generator functions.                  */
/*                                                                           */
/* Example for creating the files of factors and portfolio and calling the   */
/* macro to perform a simulation:                                            */
/*                                                                           */
/* DATA FACT;                                                                */
/*      INPUT FACTOR1-FACTOR3;                                               */
/*      CARDS;                                                               */
/*  0.002980003   0.014606266   0.003610108                                  */
/* -0.000351830  -0.006259128  -0.006255865                                  */
/* -0.001370942   0.007348310   0.021561221                                  */
/*  0.001449151  -0.001042101   0.002619011                                  */
/*  0.004008841   0.005633215   0.013214505                                  */
/* -0.002291797  -0.005394191  -0.004322111                                  */
/*  0.007452402  -0.010221110  -0.013022618                                  */
/*  0.000994247   0.008851423   0.014969136                                  */
/* -0.000343905   0.001044496   0.008742588                                  */
/*  . . .     more data lines                                                */
/* ;                                                                         */
/* RUN;                                                                      */
/* DATA PORT;                                                                */
/*      INPUT VAR LOAD1 LOAD2 LOAD3;                                         */
/*      CARDS;                                                               */
/* 0.785011716  0.352891914  0.373870389  0.273237697                        */
/* 0.811390100  0.294478292  0.384795108  0.320726600                        */
/* 0.873502876  0.118961592  0.375286443  0.505751965                        */
/* 0.883075376  0.628817260  0.016169149  0.355013591                        */
/* 0.894593499  0.221131166  0.356409292  0.422459543                        */
/* 0.014082599  0.135102802  0.429376615  0.435520583                        */
/* 0.516532135  0.293662375  0.420554950  0.285782675                        */
/* 0.073421956  0.616759924  0.051255603  0.331984474                        */
/* 0.288352860  0.093046026  0.525914520  0.381039454                        */
/* 0.124769632  0.315012854  0.132633282  0.552353864                        */
/*  . . .     more data lines                                                */
/* ;                                                                         */
/* RUN;                                                                      */
/* %EX3(FACTORS=FACT,PORT=PORT,WINDOW=45,NSIMS=10000,OUT=LOSSES,SEED=123)    */
/*                                                                           */
/*****************************************************************************/

%MACRO EX3(FACTORS=,PORT=,WINDOW=,NSIMS=,OUT=,SEED=);

 /* Create a new file that has a randomly chosen window */
 /* of market factors for each simulation.              */

 DATA TEMP(DROP=START);
      IF _N_=1 THEN DO;
         SET &FACTORS NOBS=NPERIODS;
         ARRAY FACTORS(*) _NUMERIC_;

         /* Determine the number of factors in the model, and store it in */
         /* macro variable 'NFACT'. It will be utilized at other parts of */
         /* the macro.                                                    */

         CALL SYMPUT('NFACT',LEFT(PUT(DIM(FACTORS),BEST10.)));
         CALL SYMPUT('NFACT2',LEFT(PUT(DIM(FACTORS)*DIM(FACTORS),BEST10.)));

         /* Create as many random windows as the number of simulations. */

         DO SIM=1 TO &NSIMS;

            /* Variable START is the starting observation of the window. */

            START=1+INT((NPERIODS+1-&WINDOW)*RANUNI(&SEED));
            DO I=START TO START+&WINDOW-1;
               SET &FACTORS POINT=I;
               OUTPUT;
            END;
         END;
      END;
      RUN;

 /* Determine the covariance matrix for each window. */

 PROC CORR DATA=TEMP COV NOPRINT OUT=TEMP(WHERE=(_TYPE_='COV') DROP=_NAME_);
      BY SIM;

 /* Calculate the Cholesky decomposition of each covariance matrix, */
 /* and create factor innovations with the desired correlation.     */

 DATA TEMP(KEEP=FACTOR1-FACTOR&NFACT);
      ARRAY COV(&NFACT) FACTOR1-FACTOR&NFACT;
      ARRAY COVSQ(&NFACT,&NFACT) COVSQ1-COVSQ&NFACT2;
      ARRAY CHOL(&NFACT,&NFACT) CHOL1-CHOL&NFACT2;
      ARRAY RANNUM(&NFACT) R1-R&NFACT;

      /* Perform the calculation for each window separately. */

      DO SIM=1 TO &NSIMS;

         /* Read the current covariance matrix and store it in array 'COVSQ'.*/

         START=&NFACT*(SIM-1);
         DO I=1 TO &NFACT;
            POINT=START+I;
            SET TEMP(DROP=SIM _TYPE_) POINT=POINT;
            DO J=1 TO &NFACT;
               COVSQ(I,J)=COV(J);
            END;
         END;

         /* Calculate the elements of the Cholesky matrix   */
         /* (lower triangle). It is stored in array 'CHOL'. */

         DO I=1 TO &NFACT;
            DO J=1 TO &NFACT;
               SELECT;
                 WHEN(J>I) CHOL(I,J)=0;
                 WHEN(J=I) DO; SUM=0;
                               DO K=1 TO I-1;
                                  SUM=SUM+CHOL(I,K)**2;
                               END;
                               CHOL(I,I)=SQRT(COVSQ(I,I)-SUM);
                               END;
                 OTHERWISE DO; SUM=0;
                               DO K=1 TO J-1;
                                  SUM=SUM+CHOL(I,K)*CHOL(J,K);
                               END;
                               CHOL(I,J)=(COVSQ(I,J)-SUM)/CHOL(J,J);
                               END;
               END;
            END;
         END;

         /* Utilizing the Cholesky matrix, introduce random */
         /* factors with the desired correlation.           */

         DO I=1 TO &NFACT;
            RANNUM(I)=RANNOR(&SEED);
         END;
         DO J=1 TO &NFACT;
            SUM=0;
            DO I=1 TO &NFACT;
               SUM=SUM+CHOL(I,J)*RANNUM(I);
            END;
            COV(J)=SUM;
         END;

         /* Output the random factors, one observation for each simulation. */

         OUTPUT;
      END;
      STOP;
      RUN;

 /* Determine the number of securities in the portfolio file. */
 /* Store it in macro variable 'NSECS'.                       */

 DATA _NULL_;
      IF 0 THEN SET &PORT NOBS=COUNT;
      CALL SYMPUT('NSECS',LEFT(PUT(COUNT,8.)));
      STOP;
      RUN;
 %LET PORT2=%EVAL(&NSECS*&NFACT);

 /* The main simulation section: Apply the random change to each security */
 /* of the portfolio and calculate the losses for the entire portfolio.    */

 DATA &OUT(KEEP=LOSS);

      /* Array VARS stores the variances of the securities                     */
      /* Array ALLFACTORS stores all loads of all securities along all factors */
      /*                  as a two-dimension array                             */
      /* Array FACTORS stores the factor innovations with the desired          */
      /*               correlation                                             */
      /* Array LOADS stores the loads of one security along the factors        */

      ARRAY VARS(&NSECS) VAR1-VAR&NSECS;
      ARRAY ALLFACTORS(&NFACT,&NSECS) F1-F&PORT2;
      ARRAY FACTORS(&NFACT) FACTOR1-FACTOR&NFACT;
      ARRAY LOADS(&NFACT) LOAD1-LOAD&NFACT;
      RETAIN VAR1-VAR&NSECS F1-F&PORT2;

      /* Store the loads of the securities in the portfolio. */

      SET &PORT END=EOPORT;
      VARS(_N_)=VAR;
      DO J=1 TO &NFACT;
         ALLFACTORS(J,_N_)=LOADS(J);
      END;

      /* Main simulation loop. */

      IF EOPORT THEN DO;

         DO SIM=1 TO &NSIMS;

            /* Read the factor innovations for the current simulation. */

            SET TEMP POINT=SIM;
            LOSS=0;

            /* Go through each security in the portfolio. */

            DO SEC=1 TO &NSECS;

               /* Calculate the sum of changes in the factors (variable  */
               /* DELTA) and the idiosyncratic variation (variable IDIO) */
               /* for the security in question.                          */

               IDIO=0;
               DELTA=0;
               DO J=1 TO &NFACT;
                  IDIO=IDIO+ALLFACTORS(J,SEC)**2;
                  DELTA=DELTA+VARS(SEC)*ALLFACTORS(J,SEC)*FACTORS(J);
               END;
               IDIO=VARS(SEC)*SQRT(1-IDIO)*RANNOR(&SEED);
               CHANGE=DELTA+IDIO;

               /* Summarize the loss (change) throughout the portfolio. */

               LOSS=LOSS+CHANGE;
            END;
            OUTPUT;
         END;
      END;
      RUN;

 /* Describe the loss distribution and determine certain percentiles of it. */

 PROC UNIVARIATE DATA=&OUT NOPRINT;
      VAR LOSS;
      OUTPUT OUT=RES N=N MEAN=MEAN STD=STD MAX=MAX MIN=MIN MEDIAN=MEDIAN
                     Q1=Q1 Q3=Q3 P10=P10 P5=P5 P1=P1;
      TITLE F=SWISS H=1.5
            "Distribution of Losses (Length of Window=&WINDOW., Number of Simulations=&NSIMS.)";
      RUN;

 /* Print the results in a comprehensive vertical layout. */

 PROC TRANSPOSE DATA=RES OUT=TRES;
      RUN;
 PROC PRINT DATA=TRES LABEL NOOBS;
      VAR _LABEL_ COL1;
      LABEL _LABEL_='Statistic' COL1='Loss ($)';
      FORMAT COL1 10.2;
      RUN;

 /* Draw the graph of the ordered losses. Set up the labels */
 /* for the two reference lines at 5 and 1 percentiles.     */

 DATA ANNO;
      SET RES;
      FUNCTION='LABEL';  STYLE='DUPLEX'; SIZE=1.2;
      XSYS='2'; YSYS='2';
      Y=P5; X=50; TEXT='5% '||COMPRESS(PUT(P5,10.0));
      POSITION='C'; OUTPUT;
      Y=P1; X=80; TEXT='1% '||COMPRESS(PUT(P1,10.0));
      POSITION='F'; OUTPUT;
      CALL SYMPUT('VREF5',PUT(P5,BEST10.));
      call symput('vref1',put(p1,best10.));
      run;
 PROC SORT DATA=&OUT OUT=TEMP;
      BY LOSS;
 DATA TEMP;
      SET TEMP;
      PERCENT=100*(_N_/&NSIMS);
      RUN;
 PROC GPLOT DATA=TEMP ANNOTATE=ANNO;
      PLOT LOSS*PERCENT / VAXIS=AXIS1 HAXIS=AXIS2 VREF=&VREF5 &VREF1;
      SYMBOL1 I=JOIN V=NONE W=1;
      AXIS1 LABEL=(A=90 R=0 F=SWISS H=1.5 'Portfolio loss ($)')
            VALUE=(H=1.2 F=SWISS) MINOR=(N=1) OFFSET=(0,0);
      AXIS2 LABEL=(F=SWISS H=1.5 'Percentile')
            VALUE=(H=1.2 F=SWISS) ORDER=0 TO 100 BY 10 MINOR=(N=1) OFFSET=(0,0);
      RUN;
      QUIT;
%MEND;
*******************************************************************************;
