/************************************************************************************/
/* This macro generates 200 replications and aggregates the results. As before,     */
/* Three series (Series A, B, and C) of 200 observations each with a Multivariate   */
/* AR1 process with cross lagged relationships.  The Autoregressive coefficients    */
/* for each series are .80, .60, and .45, respectively, and the error variance      */
/* components are .20, .20 and .15, respectively.  The cross lagged relationships   */
/* between Series A and B is .20; between Series A and C, .10; and between Series  */
/* B and C, .30.                                                                    */
/*                                                                                  */
/************************************************************************************/

OPTIONS LINESIZE=100 NOSOURCE NOSOURCE2 NONOTES;

LIBNAME MULTITS 'C:\MY DOCUMENTS\MY SAS FILES\RESULTS';

%MACRO MULTITS (REPS,N,VARA,AR_AA,
                       VARB,AR_BB,AR_BA,
                       VARC,AR_CC,AR_CA,AR_CB,SCENARI);

   %DO J=1 %TO &REPS;                    *** 200 replications are undertaken;

DATA GENERATE&J;
  ARRAY SERIEA SERIEA1-SERIEA&N;         *** creates space for the observations;
  ARRAY SERIEB SERIEB1-SERIEB&N;
  ARRAY SERIEC SERIEC1-SERIEC&N;
  ARRAY VAR_A  VAR_A1 -VAR_A&N ;
  ARRAY VAR_B  VAR_B1 -VAR_B&N ;
  ARRAY VAR_C  VAR_C1 -VAR_C&N ;

  SERIEA(1)=RANNOR(-1);
  SERIEB(1)=RANNOR(-11);
  SERIEC(1)=RANNOR(-111);

       VAR_A(1) =0;
       VAR_B(1) =0;
       VAR_C(1) =0;

    DO J=2 TO &N;

       VAR_A(J) =RANNOR(-1111);
       VAR_B(J) =RANNOR(-11111);
       VAR_C(J) =RANNOR(-111111);

*** generates the multivariate processes with cross-lagged relationships;

SERIEA(J)=VAR_A(J)*SQRT(&VARA) + SERIEA(J-1)*SQRT(&AR_AA);
SERIEB(J)=VAR_B(J)*SQRT(&VARB) + SERIEB(J-1)*SQRT(&AR_BB) + SERIEA(J-1)*(&AR_BA);
SERIEC(J)=VAR_C(J)*SQRT(&VARC) + SERIEC(J-1)*SQRT(&AR_CC) + SERIEA(J-1)*(&AR_CA) +
              SERIEB(J-1)*(&AR_CB);

    END; KEEP SERIEA1-SERIEA&N
             SERIEB1-SERIEB&N
             SERIEC1-SERIEC&N;
OUTPUT;

                                   *** transposes series a to prepare it for analysis;
DATA GENERAT1;SET GENERATE&J;  
   KEEP SERIEA1-SERIEA&N;PROC TRANSPOSE OUT=GENERAT1;
   DATA GENERAT1;SET GENERAT1;ID=_N_;SERIESA=COL1;OUTPUT;DROP COL1;

                                   *** transposes series b to prepare it for analysis;
DATA GENERAT2;SET GENERATE&J; 
   KEEP SERIEB1-SERIEB&N;PROC TRANSPOSE OUT=GENERAT2;
   DATA GENERAT2;SET GENERAT2;ID=_N_;SERIESB=COL1;OUTPUT;DROP COL1;

                                   *** transposes series c to prepare it for analysis;
DATA GENERAT3;SET GENERATE&J;
   KEEP SERIEC1-SERIEC&N;PROC TRANSPOSE OUT=GENERAT3;
   DATA GENERAT3;SET GENERAT3;ID=_N_;SERIESC=COL1;OUTPUT;DROP COL1;

                                   *** merges the three series by observation;

DATA GENERATE&J;MERGE GENERAT1 GENERAT2 GENERAT3;BY ID;
             KEEP ID SERIESA SERIESB SERIESC ;

                                   *** fits the multivariate model to the series data;

PROC STATESPACE INTERVAL=DAY MAXIT=200 ARMAX=1 LAGMAX=1
     NOPRINT    OUTMODEL=ARCOEFFS;

                *** outputs the collected statistics var seriesa seriesb seriesc;

      TITLE 'MULTIVARIATE ARIMA (1,0,0) CROSSLAGGED RESULTS';

                 *** plucks parameter estimates for series a out of statespace output;

DATA GENERAT1;SET ARCOEFFS;IF _N_ = 1;
      KEEP F_1 F_2 F_3 SIG_1 SIG_2 SIG_3;
      AR_AA=F_1**2;   AR_AB=F_2;   AR_AC=F_3;
      RESCOVAA=SIG_1; RESCOVAB=SIG_2; RESCOVAC=SIG_3;
      KEEP AR_AA AR_AB AR_AC RESCOVAA RESCOVAB RESCOVAC;
      OUTPUT;

                 *** plucks parameter estimates for series b out of statespace output;

DATA GENERAT2;SET ARCOEFFS;IF _N_ = 3;
      KEEP F_1 F_2 F_3 SIG_1 SIG_2 SIG_3;
      AR_BA=F_1;   AR_BB=F_2**2;   AR_BC=F_3;
      RESCOVBA=SIG_1; RESCOVBB=SIG_2; RESCOVBC=SIG_3;
      KEEP AR_BA AR_BB AR_BC RESCOVBA RESCOVBB RESCOVBC;
      OUTPUT;

                 *** plucks parameter estimates for series c out of statespace output;

DATA GENERAT3;SET ARCOEFFS;IF _N_ = 5;
      KEEP F_1 F_2 F_3 SIG_1 SIG_2 SIG_3;
      AR_CA=F_1;   AR_CB=F_2;   AR_CC=F_3**2;
      RESCOVCA=SIG_1; RESCOVCB=SIG_2; RESCOVCC=SIG_3;
      KEEP AR_CA AR_CB AR_CC RESCOVCA RESCOVCB RESCOVCC;
      OUTPUT;

                 *** merges the parameter estimates for each of the three series;

DATA ARCOEFFS; MERGE GENERAT1 GENERAT2 GENERAT3 GENERATE&J;
              SCENARIO=&SCENARI;OUTPUT;

      KEEP AR_AA AR_AB AR_AC RESCOVAA RESCOVAB RESCOVAC
           AR_BA AR_BB AR_BC RESCOVBA RESCOVBB RESCOVBC
           AR_CA AR_CB AR_CC RESCOVCA RESCOVCB RESCOVCC
           SCENARIO M_A M_B M_C VAR_A VAR_B VAR_C;

  PROC DELETE DATA=GENERATE&J;   *** frees some of the memory space for sas;
  PROC DELETE DATA=GENERAT1;
  PROC DELETE DATA=GENERAT2;
  PROC DELETE DATA=GENERAT3;

          *** adds the parameter information to an out file for one of 200 iterations;

   PROC APPEND BASE=MULTITS.RESULT (CNTLLEV=MEMBER);  

  %END;
%MEND COMPUTE;

*** the following macro statements feed the macro above the parameters for each condition under study;

%MULTITS (200, 100,  .20, .80,
                     .20, .60, .20,
                     .15, .45, .10, .30, 1);

%MULTITS (200, 500,  .20, .80,
                     .20, .60, .20,
                     .15, .45, .10, .30, 2);

%MULTITS (200, 1000, .20, .80,
                     .20, .60, .20,
                     .15, .45, .10, .30, 3);

PROC SORT;BY SCENARIO;

                           *** proc summary summarizes the results for each condition;

PROC SUMMARY PRINT VARDEF=N MAXDEC=2 FW=8; 
        CLASS SCENARIO;  *** SUMMARIZES RESULTS;
             VAR  AR_AA AR_AB AR_AC
                  AR_BA AR_BB AR_BC
                  AR_CA AR_CB AR_CC
                  RESCOVAA RESCOVAB RESCOVAC
                  RESCOVBA RESCOVBB RESCOVBC
                  RESCOVCA RESCOVCB RESCOVCC;

             TITLE1  '*****  SCENARIO 1 (N=100)  *****';
             TITLE2  '*****  SCENARIO 2 (N=500)  *****';
             TITLE3  '*****  SCENARIO 3 (N=1000) *****';

             TITLE5  'AR_AA=.80 AR_BB=.60 AR_CC=.45';
             TITLE6  'AR_BA=.20 AR_CA=.10 AR_CB=.30';
             TITLE7  ' ';
RUN;
**********************************************************************************;
